/*
    This program removes the race condition by using a semaphore
    to synchronize the main thread with the child threads. The main
    thread cannot create another child thread until the most recently
    created child thread has retrieved its (shared) parameter. This is
    not a good solution to the race condition because it serializes
    the creation of the threads. The threads are no longer really
    parallel threads.
*/
#include <stdio.h>    // printf()
#include <pthread.h>
#include <semaphore.h>

sem_t semaphore;

int main()
{
   void *thread(void *vargp);  //thread function prototype

   #define NUMBER_OF_THREADS 6
   pthread_t tid[NUMBER_OF_THREADS];
   int i;

   // initialize the (binary) semaphore (in unsignaled state)
   sem_init(&semaphore, 0, 0);

   // create threads
   printf("Starting %d threads\n", NUMBER_OF_THREADS);
   for (i = 0; i < NUMBER_OF_THREADS; i++)
   {
      pthread_create(&(tid[i]), NULL, thread, &i);
      sem_wait(&semaphore);  // wait on the most recently created thread
   }

   // wait for the threads to finish
   for (i = 0; i < NUMBER_OF_THREADS; i++)
      pthread_join(tid[i], NULL);

   printf("\nThreads stopped.\n");

   return 0;
}//main


void *thread(void *threadParam)
{
   int myParameter = *((int *)threadParam);
   sem_post(&semaphore); // signal main thread to create another child thread
   printf("Hello from thread %d\n", myParameter);
 //sem_post(&semaphore);   // try moving the sem_post to here

   return NULL;
}//thread
