/*
    This program demonstrates how a simple race condition can be
    removed.

    This program removes the race condition by removing the shared
    variable. In this case, instead of passing each thread a pointer
    to the index variable, the thread is passed a pointer into an
    array of thread parameters. Each thread gets its own slot in the
    array of thread parameters, so no thread parameters are shared.
*/
#include <stdio.h>    // printf()
#include <pthread.h>

int main()
{
   void *thread(void *vargp);  //thread function prototype

   #define NUMBER_OF_THREADS 6
   pthread_t tid[NUMBER_OF_THREADS];
   int    tParam[NUMBER_OF_THREADS] = {0, 1, 2, 3, 4, 5};
   int i;

   // create threads
   printf("Starting %d threads\n", NUMBER_OF_THREADS);
   for (i = 0; i < NUMBER_OF_THREADS; i++)
      pthread_create(&(tid[i]), NULL, thread, &(tParam[i]));

   // wait for the threads to finish
   for (i = 0; i < NUMBER_OF_THREADS; i++)
      pthread_join(tid[i], NULL);

   printf("\nThreads stopped.\n");

   return 0;
}//main


void *thread(void *threadParam)
{
   int myParameter = *((int *)threadParam);
   printf("Hello from thread %d\n", myParameter);

   return NULL;
}//thread
